# -*- coding:utf-8 -*-
from lxml import etree
import warnings, re, sys, os, time, datetime, json, pickle, configparser, requests
from colorama import init as cmd_color
warnings.filterwarnings('ignore')
cmd_color(autoreset=True)
from urllib.parse import quote,unquote
import base64

def print_color(msg, _type, end='\n'):
    print("\033[%s;%s%sm%s\033[0m" % (0, {'info': 32, 'warn': 33, 'error': 31, 'yes': 36, 'no': 35}[_type], '', msg),
          end=end)


def zsq_again_return(num=5, cback=None, sleep=0.5, show_error=True, show_filter_error=False):
    filter_error_lst = ['ProxyError', 'SSLError', 'IncompleteRead']  # 过滤错误

    def rt(func):
        def wear(*args, **keyargs):
            for i in range(num):
                try:
                    return func(*args, **keyargs)
                except Exception as e:
                    lno = e.__traceback__.tb_next.tb_lineno if e.__traceback__.tb_next else e.__traceback__.tb_lineno
                    pass_error = False
                    for item in filter_error_lst:
                        if item in str(e.args):
                            pass_error = True
                            break
                    if pass_error:
                        if show_filter_error:
                            print_color('[%s行] ERROR: %s' % (lno, e), 'error')
                    else:
                        if show_error:
                            print_color('[{}__{}行__{:_=2}次] ERROR : {}'.format(func.__name__, lno, i + 1, e),
                                        'error' if i == num - 1 else 'warn')
                    time.sleep(sleep)
            return cback

        return wear

    return rt


def zsq_try(func):
    '''    错误装饰器    '''

    def rt(*args, **keyargs):
        try:
            return func(*args, **keyargs)
        except Exception as e:
            lno = e.__traceback__.tb_next.tb_lineno if e.__traceback__.tb_next else e.__traceback__.tb_lineno
            msg = '[{}__{}行] ERROR : {}'.format(func.__name__, lno, e)
            print(msg)

    return rt


class XPATH(etree.ElementBase):
    def __init__(self, html:str):
        self.xp = etree.HTML(html)

    def title(self)->str:
        return self.xp.xpath('string(//title//text())')

    def 取首元素(self, 表达式:str)->str or etree._Element:
        '''
        第一个元素 //title
        第一个元素属性 //title/text()
        '''
        元素 = self.xp.xpath(表达式)
        if 元素:
            return 元素[0]
        else:
            return None

    def 取尾元素(self,表达式:str)->str or etree._Element:
        元素 = self.xp.xpath(表达式)
        if 元素:
            return 元素[-1]
        else:
            return None

    def 取多个元素(self, xpath_str:str)->list:
        '''
        多个元素 //img
        多个属性 //img/@src
        '''
        return self.xp.xpath(xpath_str)

    def 取文本去掉换行(self, 表达式或元素:str or etree._Element) ->str or etree._Element:
        '''
        表达式 //a
        多个元素 出入多个元素
        单个元素
        '''
        a = 表达式或元素
        if type(a) == list:
            return [x.xpath('normalize-space(.)') for x in a]
        elif type(a) == etree._Element:
            return a.xpath('normalize-space(.)')
        elif type(a) == str:
            a = self.xp.xpath(a)
            return [x.xpath('normalize-space(.)') if type(x) == etree._Element else x for x in a]

    def 取正则查询元素(self, 表达式:str, 属性:str, 正则语句:str) ->list:
        s = '{}[re:match({},"{}")]'.format(表达式, 属性, 正则语句)
        return self.xp.xpath(s, namespaces={"re": "http://exslt.org/regular-expressions"})

    def 模糊查询元素(self,表达式:str, dic:dict)->list:
        '''
        xp.模糊查询('//a', {'text()':'美女','@src':True,'@data':False})
        '''
        s=[]
        for k,v in dic.items():
            s.append('contains({},{})'.format(k,v if type(v)==bool else "\"{}\"".format(v)))
        s=' and '.join(s)
        表达式='{}[{}]'.format(表达式,s)
        return self.xp.xpath(表达式)

    def 元素集取属性(self,元素集:list,表达式:str,取首个:bool=True)->list:
        if 取首个:
            return [x.xpath('string({})'.format(表达式)) for x in 元素集]
        else:
            return [x.xpath(表达式) for x in 元素集]

    def 取同级下个元素(self,元素:etree._Element,标签:str,N=1)->etree._Element:
        表达式='following-sibling::{}[{}]'.format(标签,N)
        res=元素.xpath(表达式)
        return res[0] if res else None

    def 取同级上个元素(self,元素:etree._Element,标签:str,N=1)->etree._Element:
        表达式='preceding-sibling::{}[{}]'.format(标签,N)
        res=元素.xpath(表达式)
        return res[0] if res else None

    def 取HTML(self,元素:etree._Element,编码='utf-8')->str:
        return etree.tostring(元素,encoding=编码).decode(编码)

class conn_sqlite3():
    def __init__(self,db_path):
        self.conn = sqlite3.connect(db_path)
        self.conn.row_factory = sqlite3.Row
        self.cur = self.conn.cursor()
    def do_sql_execute(self,sql):
        try:
            self.cur.execute(sql)
            self.conn.commit()
            count = self.cur.rowcount
            return count
        except Exception as e:
            self.conn.rollback()
            raise Exception(e)
    def do_sql_select(self,sql):
        try:
            self.cur.execute(sql)
            res=self.cur.fetchall()
            return res
        except Exception as e:
            self.conn.rollback()
            raise Exception(e)
    def close(self):
        if self.conn:
            self.conn.close()

@zsq_again_return(cback=False)
def 下载文件(网址, 文件路径=None):
    if not 文件路径:
        文件路径=网址.rsplit('/',1)[-1]
    res = requests.get(网址, timeout=15, verify=False, proxies=None, allow_redirects=True).content
    with open(文件路径, 'wb') as f:
        f.write(res)
    return True


@zsq_try
def 加载文件(文件路径, 编码='utf-8-sig'):
    with open(文件路径, 'r', encoding=编码) as f:
        return f.read()


@zsq_try
def 保存文件(文件路径, 字符串, 编码='utf-8-sig'):
    with open(文件路径, 'w', encoding=编码) as f:
        f.write(s)


@zsq_try
def 加载对象(文件路径):
    with open(文件路径, 'rb') as f:
        return pickle.load(f)


@zsq_try
def 保存对象(文件路径, 对象):
    with open(文件路径, 'wb') as f:
        pickle.dump(对象, f)


@zsq_try
def 加载JSON(文件路径, 编码='utf-8-sig'):
    with open(文件路径, 'r', encoding='utf-8-sig') as f:
        return json.load(f)


@zsq_try
def 保存JSON(文件路径, JSON对象, 编码='utf-8-sig'):
    with open(文件路径, 'w', encoding=编码) as f:
        json.dump(JSON对象, f, ensure_ascii=False, indent=4)


def headStr_to_Dict(s: str) -> dict:
    head = '''%(s)s\nUser-Agent:Mozilla/5.0 (Windows NT 10.0; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/70.0.3538.25 Safari/537.36 Core/1.70.3775.400 QQBrowser/10.6.4208.400''' % {
        's': s}
    return dict([[y.strip() for y in x.strip().split(':', 1)] for x in head.strip().split('\n') if x.strip()])


def dataStr_to_Dict(s: str) -> dict:
    data = {}
    for x in s.split('&'):
        if x.strip():
            x = x.split('=', 1)
            data[x[0]] = x[1]
    return data

def 字符串转日期格式(stime,中文=False):
    if 中文:
        return '{:%Y年%m月%d日 %H时%M分%S秒}'.format(datetime.datetime.strptime(stime,"%Y-%m-%d %H:%M:%S"))
    else:
        return datetime.datetime.strptime(stime,"%Y-%m-%d %H:%M:%S")

def 时间戳转日期格式(timeStamp,中文=False):
    if 中文:
        dateArray = datetime.datetime.fromtimestamp(timeStamp)
        return dateArray.strftime("%Y年%m月%d %H时%M分%S秒")
    else:
        return datetime.datetime.utcfromtimestamp(timeStamp)

def 日期格式转时间戳():
    return datetime.datetime.today().timestamp()

def base64加密(s:str):
    return base64.b64encode(s.encode()).decode()

def base64解密(s:str):
    return base64.b64decode(s).decode()

def url编码(s):
    return quote(s)

def url解码(s):
    return unquote(s)